package com.huiju.estateagents.center.taUser.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.center.sysUser.entity.SysToken;
import com.huiju.estateagents.center.sysUser.entity.SysUser;
import com.huiju.estateagents.center.sysUser.service.ISysTokenService;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.service.ITaUserService;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.JWTUtils;
import com.huiju.estateagents.common.MD5Utils;
import com.huiju.estateagents.entity.TaPerson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
    * 用户表  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2019-09-11
 */
@RestController
@RequestMapping("/api")
public class TaUserController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaUserController.class);

    @Autowired
    public ITaUserService iTaUserService;
    
    @Autowired
    private ISysTokenService sysTokenService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/center/taUser",method= RequestMethod.GET)
    public ResponseBean taUserList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage pg = new Page<>(pageNum, pageSize);
            IPage<Map<String,Object>> result = iTaUserService.getPageList(pg);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taUser 实体对象
     * @return
     */
    @RequestMapping(value="/center/taUser",method= RequestMethod.POST)
    public ResponseBean taUserAdd(@RequestBody TaUser taUser){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaUserService.save(taUser)){
                responseBean.addSuccess(taUser);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taUser/{id}", method= RequestMethod.DELETE)
    public ResponseBean taUserDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaUserService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taUser 实体对象
     * @return
     */
    @RequestMapping(value="/taUser/{id}",method= RequestMethod.PUT)
    public ResponseBean taUserUpdate(@PathVariable Integer id,
                                        @RequestBody TaUser taUser){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaUserService.updateById(taUser)){
                responseBean.addSuccess(taUser);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 获取当前用户信息
     */
    @RequestMapping(value="/admin/taUser/current",method= RequestMethod.GET)
    public ResponseBean taUserGet(HttpServletRequest request){
        ResponseBean responseBean = new ResponseBean();
        String userId = JWTUtils.getSubject(request);
        try {
            TaUser taUser = iTaUserService.getById(userId);
            taUser.setLoginPassword("");
            responseBean.addSuccess(taUser);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taUserDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
    
    /**
     * 用户手机号登录
     * @param paramStr
     * @return
     */
    @PostMapping("/admin/taUser/signin")
    public ResponseBean signin(@RequestBody String paramStr) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (params == null) {
            return ResponseBean.error("非法参数", ResponseBean.ERROR_MISSING_PARAMS);
        }
    
        String userName = params.getString("loginName");
        String userPassword = params.getString("loginPassword");
    
        //验证用户名密码是否正确
        QueryWrapper<TaUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.eq("login_name",userName);
        sysUserQueryWrapper.eq("login_password", MD5Utils.md5(userPassword));
        TaUser taUser = iTaUserService.getOne(sysUserQueryWrapper);
        if (taUser == null) {
            return ResponseBean.error("用户名或密码错误", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
    
        if (taUser.getStatus() != CommConstant.STATUS_NORMAL) {
            return ResponseBean.error("用户状态异常", ResponseBean.ERROR_UNAVAILABLE);
        }
        taUser.setLoginPassword("");
        String token = JWTUtils.newToken(taUser.getUserId().toString());
    
        //保存token
        saveToken(token);
    
        Map<String, Object> result = new HashMap<>();
        result.put("token", token);
        result.put("user", taUser);
    
        return ResponseBean.success(result);
    }
    
    
    private void saveToken(String token) {
        SysToken sysToken = new SysToken();
        sysToken.setToken(token);
        sysToken.setExpDate(LocalDateTime.now());
        sysTokenService.save(sysToken);
    }
}
